import logging
import pyramid.httpexceptions as exc
from pyramid.httpexceptions import HTTPFound
from pyramid.response import Response
from pyramid.view import view_config, view_defaults, forbidden_view_config
from pyramid.security import remember, forget
from passlib.hash import sha256_crypt
from dryxPyramid.security import get_users_and_groups
from dryxPyramid.templates.responses import templates_login

# RESOURCE CONTEXT


@forbidden_view_config()
def forbidden(request):
    login = login_view(request)
    login.referrer = "/"
    if request.method == "GET" or ("method" in request.params and request.params["method"] == "get"):
        login.message = "You do not have the correct permissions to view this page"
    else:
        login.message = "You do not have the correct permissions to perform this action"
    # href = request.route_path('login')
    return login.login()


@view_defaults(route_name='login', permission="view_everyone")
class login_view(object):

    def __init__(self, request):
        self.request = request
        self.log = logging.getLogger(__name__)
        self.log.debug("instantiating a new 'login' view")
        self.USERS, self.GROUPS = get_users_and_groups(request)
        self.message = ""
        self.referrer = request.url

    @view_config(route_name='login', permission="view_everyone")
    def login(self):
        # Some Varibales
        request = self.request
        login_url = request.route_url('login')
        referrer = self.referrer
        message = self.message
        login = ''
        password = ''

        # never use the login form itself as came_from
        if login_url in referrer or "/" == referrer:
            href = request.route_path('index')
            referrer = href
        came_from = request.params.get('came_from', referrer)

        # test post method parameter to see if user can login
        if 'login' in request.params and (('method' in request.params and request.params["method"] == "post") or request.method == "POST"):
            login = request.params['login']
            login = login.replace("@pessto.org", "")
            password = request.params['password']
            if login not in self.USERS:
                message = 'incorrect username or password'
            else:
                if sha256_crypt.verify(password, self.USERS.get(login)):
                    headers = remember(request, login)
                    return HTTPFound(location=came_from,
                                     headers=headers)
                else:
                    message = 'incorrect username or password'

        # If wrong details added, or GET method used, return login page
        loginPage = templates_login(
            log=self.log,
            request=request,
            mainCssFilePath=self.request.registry.settings["main css filepath"],
            jsFilePath=self.request.registry.settings["main js filepath"],
            pageTitle="Login",
            iconPath=self.request.registry.settings["path to webapp icon"],
            message=message,
            came_from=came_from
        )
        responseContent = loginPage.get()
        return Response(responseContent)



#!/usr/local/bin/python
# encoding: utf-8
"""
*The HTML template module for the `templates_login.py` resource*

:Author:
    David Young

:Date Created:
    November 20, 2014

.. todo::
    
"""
################# GLOBAL IMPORTS ####################
import sys
import os
import khufu


class templates_login():

    """
    *The worker class for the templates_login module*

    **Key Arguments:**
        - ``log`` -- logger
        - ``request`` -- the pyramid request
        - ``mainCssFilePath`` -- the filename of the main css file
        - ``jsFilePath`` -- the filename of the main js file
        - ``pageTitle`` -- pageTitle
        - ``icon`` -- webapp icon
        - ``came_from`` -- the url this login page was triggered from
        - ``message`` -- message to display as notification

    .. todo::

    """
    # Initialisation

    def __init__(
        self,
        log,
        request,
        mainCssFilePath="main.css",
        jsFilePath="main-ck.js",
        pageTitle="Login",
        iconPath="",
        came_from="/",
        message=""
    ):
        self.log = log
        self.request = request
        self.mainCssFilePath = mainCssFilePath
        self.jsFilePath = jsFilePath
        self.pageTitle = pageTitle
        self.iconPath = iconPath
        self.came_from = came_from
        self.message = message

        # xt-self-arg-tmpx

        log.debug("instansiating a new 'templates_login' object")

        # Initial Actions

        return None

    def close(self):
        del self
        return None

    # Method Attributes
    def get(self):
        """
        *get the templates_login object*

        **Return:**
            - ``loginPage`` -- the login page

        .. todo::

        """
        self.log.debug('starting the ``get`` method')

        loginPage = khufu.scaffolding.login_page(
            log=self.log,
            mainCssFilePath=self.mainCssFilePath,
            jsFilePath=self.jsFilePath,
            pageTitle=self.pageTitle,
            iconPath=self.iconPath,
            came_from=self.came_from,
            message=self.message,
        )
        loginPage = loginPage.get()

        self.log.debug('completed the ``get`` method')
        return loginPage

    # xt-class-method



from pyramid.security import Allow, Everyone


class RootFactory(object):
    __acl__ = [(Allow, Everyone, 'view_everyone'),
               (Allow, 'group:view_users', ('view_everyone', 'view_users')),
               (Allow, 'group:edit_users',
                ('edit_users', 'view_everyone', 'view_users')),
               (Allow, 'group:superuser',
                ('view_everyone', 'view_users', 'edit_users', 'superuser')),
               (Allow, 'group:admin', ('view_everyone', 'view_users',
                                       'edit_users', 'superuser', 'admin')),
               (Allow, 'group:superadmin', ('view_everyone', 'view_users', 'edit_users', 'superuser', 'admin', 'superadmin'))]

    def __init__(self, request):
        pass



import logging
import pyramid.httpexceptions as exc
from pyramid.httpexceptions import HTTPFound
from pyramid.response import Response
from pyramid.view import view_config, view_defaults
from ..templates.responses import templates_download

# RESOURCE CONTEXT


@view_defaults(route_name='download', permission="view_users")
class download_view(object):

    def __init__(self, request):
        self.request = request
        self.log = logging.getLogger(__name__)
        self.log.debug("instantiating a new 'download'' view")

    @view_config(request_method='DELETE', permission="edit_users")
    @view_config(request_param="method=delete", permission="edit_users")
    def delete(self):
        return exc.exception_response(405, body_template="The DELETE method is not allowed on the 'download'")

    @view_config(request_method='PUT', permission="edit_users")
    @view_config(request_param="method=put", permission="edit_users")
    def put(self):
        return exc.exception_response(405, body_template="The PUT method is not allowed on 'download'")

    @view_config(request_method='POST', permission="edit_users")
    @view_config(request_param="method=post", permission="edit_users")
    def post(self):
        return exc.exception_response(405, body_template="The POST method is not allowed on 'download'")

    @view_config(request_method='GET', permission="view_users")
    @view_config(request_param="method=get", permission="view_users")
    def get(self):
        download = templates_download(
            log=self.log,
            request=self.request
        )
        return download.get()



# @review clean me

import logging
import pyramid.httpexceptions as exc
from pyramid.httpexceptions import HTTPFound
from pyramid.response import Response
from pyramid.view import view_config, view_defaults, forbidden_view_config
from pyramid.security import remember, forget

# RESOURCE CONTEXT


@view_config(route_name='logout',  permission="view_everyone")
def logout(request):
    referrer = request.url
    logout_url = request.route_url('logout')
    # never use the logout as came_from
    if logout_url in referrer:
        referrer = 'index'

    came_from = request.params.get('came_from', referrer)
    headers = forget(request)
    return HTTPFound(location=request.route_url(came_from),
                     headers=headers)



def get_users_and_groups(request):
    """
    *Get the users and groups from the database*
    """
    USERS = {}
    GROUPS = {}
    # get the USERS and GROUPS from the database
    sqlQuery = u"""
        select * from webapp_users
    """ % locals()
    objectDataTmp = request.db.execute(sqlQuery).fetchall()
    objectData = []
    objectData[:] = [dict(zip(row.keys(), row)) for row in objectDataTmp]

    for row in objectData:
        name = (row["firstname"] + "." + row["secondname"]).lower()
        USERS[name] = row["password"]
        GROUPS[name] = ["group:" + row["permissions"]]

    return USERS, GROUPS


def groupfinder(userid, request):
    """
    *- If the userid exists in the system, it will return a sequence of group identifiers (or an empty sequence if the user isn't a member of any groups).
    - If the userid does not exist in the system, it will return `None`.*
    """
    USERS, GROUPS = get_users_and_groups(request)

    if userid in USERS:
        return GROUPS.get(userid, [])



import io
import csv
import re
from decimal import Decimal
from datetime import datetime


class renderer_plain_text(object):

    """
    *The plain_text renderer - can return content to browser or a file to download*
    """

    def __init__(self, info):
        pass

    def __call__(self, content, system):

        request = system.get('request')
        if request is not None:
            response = request.response
            ct = response.content_type
            if ct == response.default_content_type:
                response.content_type = 'text/plain'

        # setup the file if "download" is true
        if "filename" in request.params:
            now = datetime.now()
            now = now.strftime("%Y%m%dt%H%M%S")
            filename = request.params["filename"].replace(" ", "_")
            filename = """%(filename)s_%(now)s""" % locals()
            response.content_type = 'text/plain'
            response.content_disposition = "attachment; filename=%(filename)s" % locals(
            )

        # create a virutal file to write the content to
        output = io.BytesIO(content)
        output = output.getvalue()
        output = output.strip()
        returnOutput = output

        return returnOutput



#!/usr/local/bin/python
# encoding: utf-8
"""
*The HTML template module for the `templates_download.py` resource*

:Author:
    David Young

:Date Created:
    November 13, 2014

.. todo::
    
"""
################# GLOBAL IMPORTS ####################
import sys
import os
from datetime import datetime, date, time
import khufu
from pyramid.response import FileResponse
from pyramid.path import AssetResolver


class templates_download():

    """
    *The worker class for the templates_download module*

    **Key Arguments:**
        - ``log`` -- logger
        - ``request`` -- the pyramid request
        - ``elementId`` -- the specific element requested (or False)

    .. todo::

    """

    def __init__(
        self,
        log,
        request,
        elementId=False
    ):
        self.log = log
        self.request = request
        self.elementId = elementId
        self.defaultQs = {
            "filename": "download"
        }
        self.qs = dict(self.request.params)

        log.debug("instansiating a new 'templates_download' object")

        # Initial Actions
        self._set_default_parameters()

        return None

    def close(self):
        del self
        return None

    # Method Attributes
    def get(self):
        """
        *get the templates_download object*

        **Return:**
            - ``responseContent`` -- the response

        .. todo::

        """
        self.log.debug('starting the ``get`` method')

        # build the absolute filepath for the resource
        myWebapp = AssetResolver(self.qs["webapp"])
        self.log.debug("""myWebapp: `%(myWebapp)s`""" % locals())
        url = self.request.params["url"]
        if url[0] == "/":
            url = url[1:]
        resourcePath = myWebapp.resolve(url)
        resourcePath = resourcePath.abspath()

        # start to build the response for the file
        response = FileResponse(
            resourcePath,
            request=self.request,
        )

        # change the filename of the file
        if "filename" in self.qs:
            now = datetime.now()
            now = now.strftime("%Y%m%dt%H%M%S")
            filename = self.qs["filename"].replace(" ", "_")
            filename = """%(filename)s_%(now)s""" % locals()
            response.content_disposition = "attachment; filename=%(filename)s" % locals(
            )

        self.log.debug('completed the ``get`` method')
        return response

    def _set_default_parameters(
            self):
        """
        *set default parameters*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

        """
        self.log.debug('starting the ``_set_default_parameters`` method')

        for k, v in self.defaultQs.iteritems():
            if k not in self.qs:
                self.qs[k] = v

        self.log.debug('completed the ``_set_default_parameters`` method')
        return None

    # use the tab-trigger below for new method
    # xt-class-method



import io
import datetime
import decimal
from decimal import Decimal
from pyramid.renderers import JSON
from pyramid.response import Response


class renderer_json(JSON):

    """
    *The json renderer - can return content to browser or a file to download*
    """

    def __init__(self, info):
        JSON.__init__(self, indent=4, sort_keys=True, separators=(',', ': '))

    def __call__(self, info, system):
        _render = JSON.__call__(self, info)

        if isinstance(info, list):
            info = {"data": info}

        # if table if empty
        if len(info):
            tableColumnNames = info.keys()

        self.add_adapter(datetime.datetime, self.datetime_adapter)
        self.add_adapter(decimal.Decimal, self.decimal_adapter)

        for c in tableColumnNames:
            if isinstance(info[c], float) or isinstance(info[c], long) or isinstance(info[c], Decimal):
                info[c] = float("%0.4f" % info[c])
            elif isinstance(info[c], datetime.datetime):
                thisDate = str(info[c])[:10]
                info[c] = "%(thisDate)s" % locals()

        # setup the file if "download" is true
        request = system.get('request')
        response = request.response
        if "filename" in request.params:
            now = datetime.datetime.now()
            now = now.strftime("%Y%m%dt%H%M%S")
            filename = request.params["filename"].replace(" ", "_")
            filename = """%(filename)s_%(now)s""" % locals()
            response.content_type = 'text/json'
            response.content_disposition = "attachment; filename=%(filename)s.json" % locals(
            )
        return _render(info, system)

    def datetime_adapter(self, obj, request):
        return obj.isoformat()

    def decimal_adapter(self, obj, request):
        return float(obj)



import csv
import io
import re
from decimal import Decimal
from datetime import datetime


class renderer_csv(object):

    """
    *The CSV renderer - can return plain text in browser or a file to download*
    """

    def __init__(self, info):
        pass

    def __call__(self, rows, system):
        request = system.get('request')
        if request is not None:
            response = request.response
            ct = response.content_type
            if ct == response.default_content_type:
                response.content_type = 'text/plain'

        # setup the file if "download" is true
        if "filename" in request.params:
            now = datetime.now()
            now = now.strftime("%Y%m%dt%H%M%S")
            filename = request.params["filename"].replace(" ", "_")
            filename = """%(filename)s_%(now)s""" % locals()
            response.content_type = 'text/csv'
            response.content_disposition = "attachment; filename=%(filename)s.csv" % locals(
            )

        # if table if empty
        if len(rows) == 0:
            return "no data returned"

        # Column Headers and widths
        header = []
        allRows = []
        tableColumnNames = rows[0].keys()
        columnWidths = []
        columnWidths[:] = [len(tableColumnNames[i])
                           for i in range(len(tableColumnNames))]

        # create a virutal file to write the content to
        output = io.BytesIO()
        writer = csv.writer(
            output, dialect='excel', delimiter=",", quotechar='"', quoting=csv.QUOTE_MINIMAL)

        # clean up data
        for row in rows:
            for c in tableColumnNames:
                if isinstance(row[c], float) or isinstance(row[c], long) or isinstance(row[c], Decimal):
                    row[c] = "%0.4f" % row[c]
                elif isinstance(row[c], datetime):
                    thisDate = str(row[c])[:10]
                    row[c] = "%(thisDate)s" % locals()

        # set the column widths
        for row in rows:
            for i, c in enumerate(tableColumnNames):
                if len(str(row[c])) > columnWidths[i]:
                    columnWidths[i] = len(str(row[c]))

        # fill in the data
        for row in rows:
            thisRow = []
            for i, c in enumerate(tableColumnNames):
                thisRow.append(row[c])
            allRows.append(thisRow)

        # write the headers
        for i, c in enumerate(tableColumnNames):
            header.append(c)
        writer.writerow(header)

        # write out the data
        writer.writerows(allRows)
        output = output.getvalue()

        output = output.strip()
        returnOutput = output

        return returnOutput



from setuptools import setup, find_packages
import os

moduleDirectory = os.path.dirname(os.path.realpath(__file__))
exec(open(moduleDirectory + "/dryxPyramid/__version__.py").read())


def readme():
    with open(moduleDirectory + '/README.md') as f:
        return f.read()

setup(
    name='dryxPyramid',
    version=__version__,
    description='Templating and basecode for Pyramid webapps',
    long_description=readme(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Utilities',
    ],
    keywords=['utilities', 'pyramid'],
    url='https://github.com/thespacedoctor/dryxPyramid',
    download_url='https://github.com/thespacedoctor/dryxPyramid/archive/v%(__version__)s.zip' % locals(
    ),
    author='David Young',
    author_email='davidrobertyoung@gmail.com',
    license='MIT',
    packages=find_packages(),
    package_data={'dryxPyramid': [
        'resources/*/*', 'resources/*.*']},
    include_package_data=True,
    install_requires=[
        'pyramid',
        'sqlalchemy',
        'passlib',
        'paste',
        'sqlalchemy',
        'fundamentals',
        'mod_wsgi',
        'pymysql',
        'pytest',
        'webtest'
    ],
    test_suite='nose2.collector.collector',
    tests_require=['nose2', 'cov-core'],
    # entry_points={
    #     'console_scripts': ['funniest-joke=funniest.cmd:main'],
    # },
    zip_safe=False
)
